// ----------------------------------------------
// - Orologico V2 by michele.guerra72@gmail.com -
// -         www.tattik.it (C) 2016             -
// ----------------------------------------------
// (update 26.08.2015)



// ATTENZIONE !!!!!!!!!!!!!!!!!!!!
// *******************************
// Per evitare flickering, aumentare la velocit del bus I2C da 100KHz a 800KHz
// in arduino/hardware/arduino/avr/libraries/wire/src/utility/twi.h -> TWI_FREQ 800000L
// (test su device collegati a I2C effettuato con successo)
// *******************************

// Librerie
#include "Wire.h" 		// Comunicazione I2C (fare modifica nella lib twi.h come indicato sopra!!!)
#include <dht11.h> 		// Sensore umidit
#include <RTClib.h> 	// Modulo Real Time Clock
#include <SPI.h> 		// Comunicazione seriale SPI
#include <SFE_BMP180.h> // Sensore pressione e temperatura

// Costanti
#define MAX0  		64	// Indirizzo 1o Max 6956
#define MAX1  		68	// Indirizzo 2o Max 6956
#define INTERVALLO 250	// Tempo di interrogazione dei sensori o step fase stato "macchina" (millisec)
#define FALSE		false
#define TRUE		true

// Codici tasto (esclusi Reset e Power)
#define K_PRINT		1281
#define K_FREQUENCY	1537
#define K_LEVEL		257
#define K_GENTEST	258
#define K_RECTEST	2
#define K_FUNCTION	1
#define K_INTERRUPT	4
#define K_HOLD		8
#define K_1			128
#define K_2			64
#define K_3			32
#define K_A			16
#define K_SELECTIVE	1282
#define K_AUXLEVEL	1538
#define K_STEP		514
#define K_RUN		1026
#define K_OPTIME	1025
#define K_RELTIME	1028
#define K_4			1152
#define K_5			1088
#define K_6			1056
#define K_B			1040
#define K_OPPTIME	513
#define K_OPMTIME	516
#define K_7			640
#define K_8			576
#define K_9			544
#define K_C			528
#define K_LOAD		1544
#define K_OFF		1664
#define K_0			1600
#define K_SIGN		1568
#define K_D			1552

// Led pulsanti
#define L1_PRINT		puls[3]|=1 
#define L0_PRINT		puls[3]&=~1
#define	L1_FREQUENCY	puls[3]|=2
#define	L0_FREQUENCY	puls[3]&=~2
#define L1_LEVEL		puls[3]|=8
#define L0_LEVEL		puls[3]&=~8
#define L1_GENTEST		puls[2]|=1
#define L0_GENTEST		puls[2]&=~1
#define L1_RECTEST		puls[0]|=1
#define L0_RECTEST		puls[0]&=~1
#define L1_FUNCTION		puls[0]|=2
#define L0_FUNCTION		puls[0]&=~2
#define L1_INTERRUPT	puls[2]|=2
#define L0_INTERRUPT	puls[2]&=~2	// **
#define L1_HOLD			puls[1]|=1
#define L0_HOLD			puls[1]&=~1
#define L1_1			puls[6]|=1
#define L0_1			puls[6]&=~1
#define L1_2			puls[5]|=1
#define L0_2			puls[5]&=~1
#define L1_3			puls[4]|=1
#define L0_3			puls[4]&=~1
#define L1_A			puls[7]|=1
#define L0_A			puls[7]&=~1
#define L1_SELECTIVE	led[1]|=128 // on-off da connettore E
#define L0_SELECTIVE	led[1]&=~128 // "
#define L1_AUXLEVEL		puls[3]|=4
#define L0_AUXLEVEL		puls[3]&=~4
#define L1_STEP			puls[1]|=2
#define L0_STEP			puls[1]&=~2
#define L1_RUN			puls[1]|=8
#define L0_RUN			puls[1]&=~8
#define L1_OPTIME		puls[0]|=8
#define L0_OPTIME		puls[0]&=~8
#define L1_RELTIME		puls[2]|=8
#define L0_RELTIME		puls[2]&=~8
#define L1_4			puls[6]|=2
#define L0_4			puls[6]&=~2
#define L1_5			puls[5]|=2
#define L0_5			puls[5]&=~2
#define L1_6			puls[4]|=2
#define L0_6			puls[4]&=~2
#define L1_B			puls[7]|=2
#define L0_B			puls[7]&=~2
#define L1_OPPTIME		puls[0]|=4
#define L0_OPPTIME		puls[0]&=~4
#define L1_OPMTIME		puls[2]|=4
#define L0_OPMTIME		puls[2]&=~4
#define L1_7			puls[6]|=8
#define L0_7			puls[6]&=~8
#define L1_8			puls[5]|=8
#define L0_8			puls[5]&=~8
#define L1_9			puls[4]|=8
#define L0_9			puls[4]&=~8
#define L1_C			puls[7]|=8
#define L0_C			puls[7]&=~8
#define L1_LOAD			puls[1]|=4
#define L0_LOAD			puls[1]&=~4
#define L1_OFF			puls[6]|=4
#define L0_OFF			puls[6]&=~4
#define L1_0			puls[5]|=4
#define L0_0			puls[5]&=~4
#define L1_SIGN			puls[4]|=4
#define L0_SIGN			puls[4]&=~4
#define L1_D			puls[7]|=4
#define L0_D			puls[7]&=~4


// Pin Arduino
#define TPA		 	2	// Tasto della tastiera del gruppo selezionato da MuxA0-A2
#define TPB		 	3	// "
#define TPC		 	4	// "
#define TPD		 	5	// "
#define TPE		 	6	// "
#define TPF		 	7	// "
#define TPG		 	8	// "
#define TPH		 	9	// "
#define MUXA0		10	// Multiplexer per selezione gruppo tasti della tastiera
#define MUXA1		11	// "
#define MUXA2		12	// "
#define DEBUG		13	// (debug)
#define TRESET		 0	// Ingresso lettura tasto RESET (AN0 analogico)
#define TPOWER		15	// Ingresso lettura tasto POWER (AN1 digitale = DIG15)
#define DHT11PIN    16	// Ingresso sensore umidit 	(AN2 digitale = DIG16)

// Creazione istanze
dht11 		DHT11;		// Sensore umidit
SFE_BMP180 	pressure;	// Sensore pressione (e temperatura)
RTC_DS1307 	rtc;		// Modulo Real Time Clock
DateTime	now;

// Variabili globali
unsigned long tprecedente = millis(); // Timer per gestione eventi
int 	giorno=0;		// Per modulo RTC
int 	mese=0;			// "
int 	anno=2015;		// "
int 	ora=0;			// "
int 	minuti=0;		// "
int 	secondi=0;		// "
byte 	umid=0;			// Per sensore umidit
byte	chk;			// "
int 	pressione=0;	// Per sensore pressione (e temperatura)
int 	mhg;			// "
byte 	temperatura=0;	// "
char 	status;			// "
double 	T,P,p0,a;		// "
byte	luna;			// Fase lunare (0-7)
int 	ggabs=0; 		// Giorno assoluto da 1 a 365/6
int 	ggrel=0; 		// Giorni residui da 365/6 a 1
byte	cambiodata=1;	// Indica che la data  cambiata (modifica / passaggio)
int 	d, dow;			// Per calcolo settimana 
int		dowlast;		// "
int 	satt=0; 		// Settimana attuale
int 	sres=0; 		// Settimane residue
unsigned int valore=0;	// Identificativo del tasto premuto (temp)
int		tasto=0;		// " 
bool	numflag=FALSE;	// Flag pressione e rilascio tasti del tastierino numerico
bool	keyflag=FALSE;	// Flag pressione tasti non numerici
byte	numero;			// Tasto del tastierino numerico premuto 99=nessuno 0-9=tasto
byte	cifra;			// Cifra dela valore da inserire
int		temp;			// Temporanea

// Array globali
byte key[8]  = { 0, 0, 0, 0, 0, 0, 0, 0 }; 		// Stato dei pulsanti della tastiera (1 premuto / 0 rilasciato)
byte dis[36] = { 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 
				 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 
				 15, 15, 15, 15, 15, 15 };		// Valore sui display 1-31 e A-E (0-9=numero, 10-14=-EHLP, 15 = vuoto)
byte tdis[36]; // Copia di dis[] per memo dati temporanei durante lampeggio
byte puls[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };    	// Stato dei led della tastiera (escluso SELECTIVE) (0/1 = off/on )
byte led[9]  = { 0, 0, 0, 0, 0, 0, 0, 0, 0 }; 	// Stato dei led sul pannello (+ SELECTIVE)
byte gg[12]  = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }; // Giorni dei mesi

// Stato e fase della "macchina"
byte stato = 0; // Stato
byte fase  = 0; // Fase


// Inizializzazione
void setup()
{
	// Configurazione pin digitali DIG2-DIG9 (DIG0-DIG1 non usati)
	// - Ingressi con pullup attivo relativi al gruppo di pulsanti della tastiera selezionato da Mux A0-A2
	pinMode(TPA, INPUT); digitalWrite(2, HIGH); // TPA 
	pinMode(TPB, INPUT); digitalWrite(3, HIGH); // TPB
	pinMode(TPC, INPUT); digitalWrite(4, HIGH); // TPC
	pinMode(TPD, INPUT); digitalWrite(5, HIGH); // TPD
	pinMode(TPE, INPUT); digitalWrite(6, HIGH); // TPE
	pinMode(TPF, INPUT); digitalWrite(7, HIGH); // TPF
	pinMode(TPG, INPUT); digitalWrite(8, HIGH); // TPG
	pinMode(TPH, INPUT); digitalWrite(9, HIGH); // TPH
	// - Uscite
	pinMode(MUXA0, OUTPUT); // Mux A0 per selezione gruppo tasti tastiera
	pinMode(MUXA1, OUTPUT); // Mux A1 "
	pinMode(MUXA2, OUTPUT); // Mux A2 "
	pinMode(DEBUG, OUTPUT); // (debug)
	
	// Configurazione pin analogici AN0-AN2 usati come ingressi digitali con pullup attivo
	pinMode(TRESET,   INPUT); digitalWrite(TRESET,   HIGH); // AN0 -> DIG14: tasto Power
	pinMode(TPOWER,   INPUT); digitalWrite(TPOWER,   HIGH); // AN1 -> DIG15: tasto Reset
	pinMode(DHT11PIN, INPUT); digitalWrite(DHT11PIN, HIGH); // AN2 -> DIG16: sensore umidit DHT11
	
	// Attivazione comunicazione I2C
	Wire.begin();
	
	// Configurazione MAX6956
	iWrite(MAX0,  4,  0);	// spegnimento
	iWrite(MAX0,  9, 85); 	// p04-p07 (non presente versione 28 pin)
	iWrite(MAX0, 10, 85); 	// p08-p11 (non presente versione 28 pin)
	iWrite(MAX0, 11, 85); 	// p12-p15 output
	iWrite(MAX0, 12, 85); 	// p16-p19 output
	iWrite(MAX0, 13, 85); 	// p20-p23 output
	iWrite(MAX0, 14, 85); 	// p24-p27 output
	iWrite(MAX0, 15, 85); 	// p28-p31 output
	iWrite(MAX0,  2, 15); 	// corrente per tutti = 16/16 iref
	iWrite(MAX0, 76,  0);   // azzera le uscite P12-P19
	iWrite(MAX0, 84,  0);   // P20-P27
	iWrite(MAX0, 92,  0);   // P28-P31
	iWrite(MAX0,  4,  1);   // attivazione

	iWrite(MAX1,  4,  0);   // spegnimento
	iWrite(MAX1,  9, 85); 	// p04-p07 (non presente versione 28 pin)
	iWrite(MAX1, 10, 85); 	// p08-p11 (non presente versione 28 pin)
	iWrite(MAX1, 11, 85); 	// p12-p15 output
	iWrite(MAX1, 12, 85); 	// p16-p19 output
	iWrite(MAX1, 13, 85); 	// p20-p23 output
	iWrite(MAX1, 14, 85); 	// p24-p27 output
	iWrite(MAX1, 15, 85); 	// p28-p31 output
	iWrite(MAX1, 02, 15); 	// corrente per tutti = 16/16 iref
	iWrite(MAX1, 76,  0); 	// azzera le uscite P12-P19
	iWrite(MAX1, 84,  0); 	// P20-P27
	iWrite(MAX1, 92,  0); 	// P28-P31
	iWrite(MAX1,  4,  1);   // attivazione

	// ******* Solo per debug ***********
	Serial.begin(115200); // debug
	
	// Verifica impostazioni modulo RTC
	if (!rtc.isrunning())
	{
		 // Impostazione solo in caso di inizializzazione forzata
		 // anno, mese, giorno, ore, minuti, secondi -> 12:00:00 del 01.01.2015
		 rtc.adjust(DateTime(2015, 1, 1, 12, 0, 0));
	}
	
	// Anno bisestile -> Febbraio = 29 giorni
	DateTime now = rtc.now();
	if ((now.year() % 4) == 0) gg[1]= 29;
	
	// Avvio sensore pressione (e temperatura) per acquisizione dati di calibrazione
	if (pressure.begin()) Serial.println("BMP180 OK"); else Serial.println("BMP180 FAIL!");

	// Modifica configurazione variabili display e led per avere una corretta impostazione
	// binaria in base alla configurazione dei byte da inviare ai MAX6956
	// la variabile  sempre mantenuta configurata correttamente ed il calcolo *16+15 o *8 non viene
	// eseguito nella routine di aggiornamento per risparmiare tempo di calcolo a favore del tempo di visualizzazione
	for (byte i=0; i<36; i++)
	{
   	    if (i>15) dis[i]=dis[i]*16+15; // *16+15 (*8) per corretta configurazione binaria dei display
		if (i>30) dis[i]=dis[i]*8;
	}
	led[0]= 16 | 15; // 16= Absolute+Relative OFF, 0=Abs ON, 32=Abs+Rel ON, 64=Rel ON (+15 per config. binaria ok)
}


// Ciclo di funzionamento
void loop()
{
	// Tasto POWER rilasciato
	if (digitalRead(TPOWER) && stato>0) // se non  premuto e prima lo era...
	{ 
		stato = 0; // Rilasciato: Tutti i display e led spenti, solo gioco led power+standby
		fase  = 0;

		// Azzera led e display
		for (byte i=0; i<36; i++)
		{
			dis[i]=15;
			if (i>15) dis[i]=dis[i]*16+15; // *16+15 (*8) per corretta configurazione binaria dei display
			if (i>30) dis[i]=dis[i]*8;
			if (i<9) led[i]=0;
			if (i<8) puls[i]=0;
		}
		led[0]= 16 | 15; // 16= Absolute+Relative OFF, 0=Abs ON, 32=Abs+Rel ON, 64=Rel ON (+15 per config. binaria ok)	
	}
	
	// Tasto POWER premuto
	if (!digitalRead(TPOWER) && stato==0) // se  premuto e prima era rilasciato...
	{
		stato = 1; // premuto: funzionamento normale
		fase  = 0;
		led[8]= 2; // spegne led standby
		for (int i=0; i<8; i++) puls[i]=0; // spegne led pulsanti
		L1_LOAD; // LOAD acceso
	}
	
	// Verifica pressione altri tasti (controlla se  stato rilasciato prima il tasto precedente)
	tasto=0; temp=0;
	for (unsigned int i=0; i<8; i++)
	{
		digitalWrite(10, bitRead(i,0)); // mux a0
		digitalWrite(11, bitRead(i,1)); // mux a1 
		digitalWrite(12, bitRead(i,2)); // mux a2
		
		valore = B11111111 ^ (((PINB & B00000011)<<6) + ((PIND & B11111100)>>2));
		
		byte c=0;
		for (byte j=0; j<8; j++) c+=bitRead(valore,j);
		// Se c>1 sono stati premuti pi tasti contemporaneamente
		if (valore>0 && c==1 && keyflag==FALSE) { tasto=(i*256+valore); keyflag=TRUE; }
		temp+=valore;
	}
	
	// Verifica rilascio: finch il tasto non  rilasciato non  pi rilevato di nuovo
	if (temp==0) keyflag=FALSE;
			
	// Codifica la pressione di un tasto numerico (verifica il rilascio prima di segnalare un'altra pressione)
	numero=99;
	if (numflag==FALSE)
	{
		if (tasto==K_1) numero=1; 
		if (tasto==K_2) numero=2;
		if (tasto==K_3) numero=3;
		if (tasto==K_4) numero=4;
		if (tasto==K_5) numero=5;
		if (tasto==K_6) numero=6;
		if (tasto==K_7) numero=7;
		if (tasto==K_8) numero=8;
		if (tasto==K_9) numero=9;
		if (tasto==K_0) numero=0;
		if (tasto==K_1 || tasto==K_2 || tasto==K_3 || tasto==K_4 || tasto==K_5 ||
		    tasto==K_6 || tasto==K_7 || tasto==K_8 || tasto==K_9 || tasto==K_0 ) numflag=TRUE;
	}
	
	// Verifica il rilascio dei tasti numerici
	if (tasto!=K_1 && tasto!=K_2 && tasto!=K_3 && tasto!=K_4 && tasto!=K_5 &&
		tasto!=K_6 && tasto!=K_7 && tasto!=K_8 && tasto!=K_9 && tasto!=K_0 ) numflag=FALSE;
		
	// Gestione eventi a seconda della pressione dei tasti
	if (stato==1) // normale
	{
		if (tasto==K_LOAD) { stato=2; fase=0; }  // sequenza lampeggi per inserimento data/ora
	}

	if (stato==2) // gi premuto LOAD
	{
		if (tasto==K_FREQUENCY) stato=3; // modifica giorno
		if (tasto==K_LEVEL    ) stato=4; // modifica mese
		if (tasto==K_GENTEST  ) stato=5; // modifica anno
		if (tasto==K_FUNCTION ) stato=6; // modifica ora
		if (tasto==K_INTERRUPT) stato=7; // modifica minuti
		if (stato>2) 
		{ 
			fase=0; cifra=1; 
			dis[2]=tdis[2]; dis[3]=tdis[3]; // attiva tutti i display che potrebbero essere spenti
			dis[4]=tdis[4]; dis[5]=tdis[5]; // (dipende da valore della variabile fase nel mmomento della pressione dei tasti di selezione)
			dis[8]=tdis[8]; dis[9]=tdis[9];
			dis[31]=tdis[31]; dis[32]=tdis[32];
			dis[34]=tdis[34]; dis[35]=tdis[35];
			L1_FREQUENCY; L1_LEVEL; L1_GENTEST; L1_FUNCTION; L1_INTERRUPT; 
		}
	}

	if (stato==3) // Set GIORNO sui display
	{	
		if (numflag==TRUE && numero>=0 && numero<=9)
		{
			if (cifra==1) { tdis[2]=numero; cifra=2; } else { tdis[3]=numero; cifra=1; }
		}
		
		if (tasto==K_LOAD) 
		{
			DateTime now = rtc.now(); // memo se entro limiti mese attuale
			if ((tdis[2]*10+tdis[3])>=1 && (tdis[2]*10+tdis[3])<=gg[now.month()-1]) 
			{ 
				dis[2]=tdis[2]; dis[3]=tdis[3]; 
			}
			else
			{
				if ((tdis[2]*10+tdis[3])<1 ) { dis[2]=0; dis[3]=1; }
				if ((tdis[2]*10+tdis[3])>31) { dis[2]=gg[now.month()-1]/10; dis[3]=gg[now.month()-1]%10; }
			}

			// anno, mese, giorno, ore, minuti, secondi
			rtc.adjust(DateTime(now.year(), now.month(), dis[2]*10+dis[3], now.hour(), now.minute(), 0));
		
			stato=0; fase=0; // ritorna allo stato normale
		}

		// Esce inserimento dati
		if (tasto==K_OFF) { stato=0; fase=0; }
	}
		
	if (stato==4) // Set MESE sui display
	{	
		if (numflag==TRUE && numero>=0 && numero<=9)
		{
			if (cifra==1) { tdis[4]=numero; cifra=2; } else { tdis[5]=numero; cifra=1; }
		}
		
		if (tasto==K_LOAD) 
		{
			DateTime now = rtc.now(); 
			if ((tdis[4]*10+tdis[5])>=1 && (tdis[4]*10+tdis[5])<=12) 
			{ 
				dis[4]=tdis[4]; dis[5]=tdis[5]; 
			}
			else
			{
				if ((tdis[4]*10+tdis[5])<1 ) { dis[4]=0; dis[5]=1; }
				if ((tdis[4]*10+tdis[5])>12) { dis[4]=1; dis[5]=2; }
			}

			if (now.day()>gg[dis[4]*10+dis[5]-1]) // il giorno attuale  superiore ai giorni del mese impostato
			{
				// imposta il giorno del mese al numero dei giorni del mese impostato
				rtc.adjust(DateTime(now.year(), dis[4]*10+dis[5], gg[dis[4]*10+dis[5]-1], now.hour(), now.minute(), 0));
			}
			else
			{
				// anno, mese, giorno, ore, minuti, secondi
				rtc.adjust(DateTime(now.year(), dis[4]*10+dis[5], now.day(), now.hour(), now.minute(), 0));
			}
			
			stato=0; fase=0; // ritorna allo stato normale
		}

		// Esce inserimento dati
		if (tasto==K_OFF) { stato=0; fase=0; }
	}
	
	if (stato==5) // Set ANNO sui display
	{	
		if (numflag==TRUE && numero>=0 && numero<=9)
		{
			if (cifra==1) { tdis[8]=numero; cifra=2; } else { tdis[9]=numero; cifra=1; }
		}
		
		if (tasto==K_LOAD) 
		{
			DateTime now = rtc.now(); 
			dis[8]=tdis[8]; dis[9]=tdis[9]; 
			// anno, mese, giorno, ore, minuti, secondi
			rtc.adjust(DateTime(2000+dis[8]*10+dis[9], now.month(), now.day(), now.hour(), now.minute(), 0));
			
			stato=0; fase=0; // ritorna allo stato normale
		}

		// Esce inserimento dati
		if (tasto==K_OFF) { stato=0; fase=0; }
	}
	
	if (stato==6) // Set ORA sui display
	{	
		if (numflag==TRUE && numero>=0 && numero<=9)
		{
			if (cifra==1) { tdis[31]=numero*8; cifra=2; } else { tdis[32]=numero*8; cifra=1; }
		}
		
		if (tasto==K_LOAD) 
		{
			tdis[31]=tdis[31]/8; tdis[32]=tdis[32]/8; // /8 elimina config binaria usata per la visualizzazione
			
			DateTime now = rtc.now(); 
			if ((tdis[31]*10+tdis[32])<=23) 
			{ 
				dis[31]=tdis[31]; dis[32]=tdis[32]; 
			}
			else
			{
				dis[31]=2; dis[32]=3; 
			}

			// anno, mese, giorno, ore, minuti, secondi
			rtc.adjust(DateTime(now.year(), now.month(), now.day(), dis[31]*10+dis[32], now.minute(), 0));
			
			stato=0; fase=0; // ritorna allo stato normale
		}

		// Esce inserimento dati
		if (tasto==K_OFF) { stato=0; fase=0; }
	}
	
	if (stato==7) // Set MINUTI sui display
	{	
		if (numflag==TRUE && numero>=0 && numero<=9)
		{
			if (cifra==1) { tdis[34]=numero*8; cifra=2; } else { tdis[35]=numero*8; cifra=1; }
		}
		
		if (tasto==K_LOAD) 
		{
			tdis[34]=tdis[34]/8; tdis[35]=tdis[35]/8; // /8 elimina config binaria usata per la visualizzazione
			
			DateTime now = rtc.now(); 
			if ((tdis[34]*10+tdis[35])<=59) 
			{ 
				dis[34]=tdis[34]; dis[35]=tdis[35]; 
			}
			else
			{
				dis[34]=5; dis[35]=9;
			}

			// anno, mese, giorno, ore, minuti, secondi
			rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), dis[34]*10+dis[35], 0));
			
			stato=0; fase=0; // ritorna allo stato normale
		}

		// Esce inserimento dati
		if (tasto==K_OFF) { stato=0; fase=0; }
	}
	
	// Comandi da eseguire ad intervalli prestabiliti
	if (millis()-tprecedente>INTERVALLO)
	{
		tprecedente=millis(); // riaggiorna istante di riferimento
	
		// Spezzare in fasi le letture ed i calcoli diminuisce l'eventuale flickering dei display/led.
		// Infatti, pi corte sono le fasi, minore sar il tempo in cui non viene eseguito il refresh dei led.
		fase++;
	
		// Tasto POWER rilasciato
		if (stato==0)
		{
			if (fase==1) led[8]=1; // Lampeggio alternato dei led Standby e Power
			if (fase==4) led[8]=2; 
			if (fase>=8) fase=0;
		}
		
		// Lettura periodica dell'RTC e dei sensori ed aggiornamento dei valori
		if (stato==1)
		{
			// Lettura sensori e periferiche (orario, temperatura, umidit) e calcoli correlati
			// La lettura dei sensori su I2C provoca un rallentamento nell'esecuzione continua
			// della routine di aggiornamento dei led e dei display, provocando una breve
			// mancata visualizzazione delle info sul pannello. Spezzando gli aggiornamenti non
			// si risolve il problema, in quando il tempo maggiore impiegato per la lettura dei
			// sensori  quello del sensore di umidit, mentre gli altri hanno letture pi veloci.
			if (fase==1)
			{
				// Lettura orario
				now    = rtc.now();
				giorno = now.day();
				mese   = now.month();
				anno   = now.year();
			
				// Lettura umidit
				chk = DHT11.read(DHT11PIN);
				if (chk==0) umid=DHT11.humidity; else umid=0; 
								
				// Aggiornamento display Umidit
				dis[28]=(umid/100)*16+15; 
				dis[29]=((umid/10)%10)*16+15; 
				dis[30]=(umid%10)*16+15;
				
				// Lettura pressione e temperatura
				temperatura=0; pressione=0; mhg=0;
				
				// Procedura  prevista dal costruttore per il sensore utilizzato
				status = pressure.startTemperature();
				if (status != 0)
				{
					delay(status); // Wait for the measurement to complete:
					status = pressure.getTemperature(T); 	  // Retrieve the completed temperature measurement. Function returns 1 if successful, 0 if failure.
					if (status != 0)
					{
						temperatura=T; // temperatura in gradi centigradi, diminuisce precisione da double a int

						// Start a pressure measurement:
						// The parameter is the oversampling setting, from 0 to 3 (highest res, longest wait).
						// If request is successful, the number of ms to wait is returned.
						// If request is unsuccessful, 0 is returned.

						status = pressure.startPressure(3);
						if (status != 0)
						{
							// Wait for the measurement to complete:
							delay(status);

							// Retrieve the completed pressure measurement:
							// Note that the measurement is stored in the variable P.
							// Note also that the function requires the previous temperature measurement (T).
							// (If temperature is stable, you can do one temperature measurement for a number of pressure measurements.)
							// Function returns 1 if successful, 0 if failure.
							
							status = pressure.getPressure(P,T);
							if (status != 0)
							{
								pressione=P; // diminuisce la precisione da double a int
								mhg=pressione/1.33; // Fattore di conversione 1013,25 hPa = 760 mmHg
							}
						}
					}
				}
				
				// Aggiornamento display e led 	*** Sezione PARAMETRI ***
				
				// Pressione in hPa
				dis[10]=pressione/1000; dis[11]=(pressione/100)%10;
				dis[12]=(pressione/10)%10; dis[13]=pressione%10;
				
				//Pressione in mmHg
				dis[14]=mhg/1000; dis[15]=(mhg/100)%10; dis[16]=((mhg/10)%10)*16+15; dis[17]=(mhg%10)*16+15;
				
				// Temperatura
				dis[25]=(temperatura/100)*16+15; 
				dis[26]=((temperatura/10)%10)*16+15; 
				dis[27]=(temperatura%10)*16+15;
								
				// Fase lunare
				luna= 1 + moon_phase(anno, mese, giorno); // 1=nuova, 5=full, 8=calante del tutto
				if (luna<1 || luna>8) luna=11; // 11=E su display (errore)
				
				// Giorno dell'anno
				ggabs=0; // giorno assoluto da 1 a 365/6
				for (int i=1; i<mese; i++) ggabs=ggabs+gg[i-1];
				ggabs=ggabs+giorno;
				ggrel=((gg[1]==29) ? 366 : 365) - ggabs; // Se anno bisestile calcola residuo da 366 giorni totali
				
				// Calcolo giorno della settimana attuale (dom-sat)
				// (fonte: http://stackoverflow.com/questions/6054016/c-program-to-find-day-of-week-given-date)
				d=giorno;
				for (int i = 1900; i<anno; i++)
					if (i%4 == 0 && (i%100 != 0 || i%400 == 0)) d += 366; else d += 365;
				for (int i = 0; i < mese - 1; i++) d += gg[i];
				dow = d%7; // giorno della settimana = 0: Sun, 1: Mon ... 6: Sat
				
				d=31; // calcolo giorno della settimana (dom-sat) del 31.12 dell'anno precedente
				for (int i = 1900; i < anno-1; i++)
					if (i%4 == 0 && (i%100 != 0 || i%400 == 0)) d += 366; else d += 365;
				for (int i = 0; i < 12 - 1; i++) d += gg[i];
				dowlast = d%7; // giorno della settimana = 0: Sun, 1: Mon ... 6: Sat
				
				// Settimana attuale e residue
				satt = (ggabs/7)+1;	if (dow<dowlast) satt++;
				sres = 53 - satt;
				
				// Aggiornamento display e led 	*** Sezione DATA ***
				
				// Fase lunare
				dis[0]=13; dis[1]=luna; // "L" + fase lunare
				
				// Giorno, mese e anno
				dis[2]=giorno/10; dis[3]=giorno%10;
				dis[4]=mese/10;   dis[5]=mese%10;
				dis[6]=anno/1000; dis[7]=(anno/100)%10; dis[8]=(anno/10)%10; dis[9]=anno%10;
								
				// Settimana attuale e residue
				dis[21]=(satt/10)*16+15; dis[22]=(satt%10)*16+15;
				dis[23]=(sres/10)*16+15; dis[24]=(sres%10)*16+15;
			}
		
			// Aggiornamento display e led 	*** Sezione ORA ***
			if (fase!=0)
			{
				// Lettura orario
				now    = rtc.now();
				ora    = now.hour(); 
				minuti = now.minute();
				secondi= now.second(); 
				giorno = now.day();
				mese   = now.month();
				anno   = now.year();
				
				// Orario
				dis[31]=(ora/10)*8; // *8 per config corretta byte
				dis[32]=(ora%10)*8;
				dis[33]=10*8; // trattino
				dis[34]=(minuti/10)*8;
				dis[35]=(minuti%10)*8;
				
				// Giorno, mese e anno
				dis[2]=giorno/10; dis[3]=giorno%10;
				dis[4]=mese/10;   dis[5]=mese%10;
				dis[6]=anno/1000; dis[7]=(anno/100)%10; dis[8]=(anno/10)%10; dis[9]=anno%10;
				
				
				// Led del pannello usati per indicazione di secondi e minuti
				if (secondi>=0 && secondi<= 7) { led[4]=0; led[5]= 8; } // 1 led per ogni 7 o 8 secondi
				if (secondi> 7 && secondi<=14) { led[4]=0; led[5]=16; }
				if (secondi>14 && secondi<=21) { led[4]=0; led[5]=32; } 
				if (secondi>21 && secondi<=29) { led[4]=0; led[5]=64; }
				if (secondi>29 && secondi<=37) { led[4]=0; led[5]= 4; }
				if (secondi>37 && secondi<=44) { led[4]=0; led[5]= 2; }
				if (secondi>44 && secondi<=51) { led[4]=0; led[5]= 1; } 
				if (secondi>51 && secondi<=59) { led[4]=128; led[5]=0; }
				
				if (minuti>=0 && minuti<=19) { led[3]=0; led[2]=128; } // 1 led ogni 20 minuti
				if (minuti>19 && minuti<=39) { led[3]=1; led[2]=0; }
				if (minuti>39 && minuti<=59) { led[3]=2; led[2]=0; }
				
				// Led vari in base al valore dei secondi
				if (secondi%2==0) led[7]|= 1;  else led[7]&=~1; // "Gen Number"
				if (secondi%3==0) led[7]|= 2;  else led[7]&=~2; // "Rec. Combination"
				if (secondi%5==0) led[3]|=16;  else led[3]|=8;  // "ACCEPTED" o "REJECTED"
				if (secondi%7==0) led[6]|=128; else led[6]&=~128; // "REMOTE"
			}
			
			// Visualizzazione alternata abs/rel giorno dell'anno
			if ((fase%8)==0 || (fase==1)) 
			{
				// display 18-20 attenzione *16+15
				dis[18]=(ggabs/100)*16+15; 
				dis[19]=((ggabs/10)%10)*16+15; 
				dis[20]=(ggabs%10)*16+15;
				led[0]= 0 | 15; // 16= Absolute+Relative OFF, 0=Abs ON, 32=Abs+Rel ON, 64=Rel ON (+15 per config. binaria ok)
			}
			
			if ((fase%16)==0) 
			{
				// display 18-20 attenzione *16+15
				dis[18]=(ggrel/100)*16+15; 
				dis[19]=((ggrel/10)%10)*16+15; 
				dis[20]=(ggrel%10)*16+15;
				led[0]= 64 | 15; // 16= Absolute+Relative OFF, 0=Abs ON, 32=Abs+Rel ON, 64=Rel ON (+15 per config. binaria ok)
			}
			
			if (fase>=120) fase=0; // restart ciclo fasi dopo 120*0,25 sec = 30 secondi
			
			// Attiva effetti led ogni 10 secondi solo in funzionamento normale
			if (secondi==10) { stato=8; fase=0; }
		}
	
		// Ciclo di illuminazioni impulsive per spiegare quali tasti premere per cambiare
		// determinati valori (giorno, mese, anno, ore, minuti)
		if (stato==2)
		{
						
			if (fase==1) 
			{ 
				tdis[2]=dis[2]; tdis[3]=dis[3]; // giorno
				tdis[4]=dis[4]; tdis[5]=dis[5]; // mese
				tdis[8]=dis[8]; tdis[9]=dis[9]; // anno
				tdis[31]=dis[31]; tdis[32]=dis[32]; // ore
				tdis[34]=dis[34]; tdis[35]=dis[35]; // minuti
				L1_FREQUENCY; L1_LEVEL; L1_GENTEST; L1_FUNCTION; L1_INTERRUPT; 
				L0_LOAD;
			} 
			
			// giorno
			if (fase== 2 || fase== 4 || fase== 6 || fase== 8) { dis[2]=tdis[2];   dis[3]=tdis[3];   L1_FREQUENCY; } // on
			if (fase== 3 || fase== 5 || fase== 7)             { dis[2]=15;        dis[3]=15;        L0_FREQUENCY; } // off
			// mese
			if (fase== 9 || fase==11 || fase==13 || fase==15) { dis[4]=tdis[4];   dis[5]=tdis[5];   L1_LEVEL;     } // on
			if (fase==10 || fase==12 || fase==14)             { dis[4]=15;        dis[5]=15;        L0_LEVEL;     } // off
			// anno
			if (fase==16 || fase==18 || fase==20 || fase==22) { dis[8]=tdis[8];   dis[9]=tdis[9];   L1_GENTEST;   } // on
			if (fase==17 || fase==19 || fase==21)             { dis[8]=15;        dis[9]=15;        L0_GENTEST;   } // off
			// ora
			if (fase==24 || fase==26 || fase==28 || fase==30) { dis[31]=tdis[31]; dis[32]=tdis[32]; L1_FUNCTION;  } // on
			if (fase==25 || fase==27 || fase==29)             { dis[31]=15*8;     dis[32]=15*8;     L0_FUNCTION;  } // off
			// minuti
			if (fase==32 || fase==34 || fase==36 || fase==38) { dis[34]=tdis[34]; dis[35]=tdis[35]; L1_INTERRUPT; } // on
			if (fase==33 || fase==35 || fase==37)             { dis[34]=15*8;     dis[35]=15*8;     L0_INTERRUPT; } // off
			
			// Esce dalla fase di modifica dei valori ~10 secondi dopo i lampeggi
			if (fase>=120) 
			{ 
				stato=1; fase=0; 
				L0_FREQUENCY; L0_LEVEL; L0_GENTEST; L0_FUNCTION; L0_INTERRUPT; 
				L1_LOAD;
			} 
		
		}
		
		// Lampeggi delle sezioni display
		if (stato==3 || stato==4 || stato==5 || stato==6 || stato==7)
		{
			// Illumina tastierino
			if (fase==1)
			{
				L1_1; L1_2; L1_3; L1_4; L1_5; L1_6; L1_7; L1_8; L1_9; L1_0;
				L1_LOAD; L1_OFF;
			}

			if (stato==3) // Giorno
			{
				if (fase== 2 || fase== 4 || fase== 6 || fase== 8) { dis[2]=tdis[2]; dis[3]=tdis[3]; L1_FREQUENCY; } // on
				if (fase== 3 || fase== 5 || fase== 7)             { dis[2]=15;      dis[3]=15;      L0_FREQUENCY; } // off
			}
			
			if (stato==4) // Mese
			{
				if (fase== 2 || fase== 4 || fase== 6 || fase== 8) { dis[4]=tdis[4]; dis[5]=tdis[5]; L1_LEVEL; } // on
				if (fase== 3 || fase== 5 || fase== 7)             { dis[4]=15;      dis[5]=15;      L0_LEVEL; } // off
			}
			if (stato==5) // Anno
			{
				if (fase== 2 || fase== 4 || fase== 6 || fase== 8) { dis[8]=tdis[8]; dis[9]=tdis[9]; L1_GENTEST; } // on
				if (fase== 3 || fase== 5 || fase== 7)             { dis[8]=15;      dis[9]=15;      L0_GENTEST; } // off
			}
			if (stato==6) // Ora
			{
				if (fase== 2 || fase== 4 || fase== 6 || fase== 8) { dis[31]=tdis[31]; dis[32]=tdis[32]; L1_FUNCTION; } // on
				if (fase== 3 || fase== 5 || fase== 7)             { dis[31]=15*8;     dis[32]=15*8;     L0_FUNCTION; } // off
			}
			if (stato==7) // Minuto
			{
				if (fase== 2 || fase== 4 || fase== 6 || fase== 8) { dis[34]=tdis[34]; dis[35]=tdis[35]; L1_INTERRUPT; } // on
				if (fase== 3 || fase== 5 || fase== 7)             { dis[34]=15*8;     dis[35]=15*8;     L0_INTERRUPT; } // off
			}

			if (fase>8) fase=1;
		}

		// Effetti led pulsanti (stato=8 attivato da stato=1 quando secondi=10)
		if (stato==8)
		{
			if (fase==0) L0_LOAD;
			if (fase==1 || fase==11 || fase==21 || fase==31)
			{
				for (int i=0; i<8; i++) puls[i]=random(0,15);
				L0_LOAD;
			}
			
			if (fase>=41) 
			{
				for (int i=0; i<8; i++) puls[i]=0;
				L1_LOAD;
				stato=0; fase=0;
			}
		}
	}

	// Abilitazione dei display e dei led con timeframe completo in 32 step
	// (solo se tasto power premuto)
	if (stato!=1000) 
	{
		// Per il significato delle abilitazioni consultare la documentazione allegata al progetto
		// (documento Excel "Temporizzazioni da t0 a t31")
		
		//		  *-------------max0----------------------*   *-----------------------------max1----------------*
        //         *--76--*             *--84--*   *--92--*   *--76--*              *--84--*  *--92--*
		//         76543210             76543210   76543210   76543210   		              76543210
		isixWrite(B11110000 |  dis[0], B00010000, B00000000, led[1] | puls[0]     , dis[31], B00000100 | led[8]); // 0 <-Frame
		isixWrite(B11110000 |  dis[1], B00010001, -1       , -1                   , dis[31], B00000100 | led[8]); // 1
		isixWrite(B11110000 |  dis[2], B00010010, -1       , -1                   , dis[31], B00000100 | led[8]); // 2
		isixWrite(B11110000 |  dis[3], B00010011, -1       , -1                   , dis[31], B00000100 | led[8]); // 3
        //         *--76--*             *--84--*   *--92--*   *--76--*              *--84--*  *--92--*
		isixWrite(B11110000 |  dis[4], B00100100, -1       ,     16 | puls[1]     , dis[32], B00000100 | led[8]); // 4
		isixWrite(B11110000 |  dis[5], B00100101, -1       , -1                   , dis[32], B00000100 | led[8]); // 5
		isixWrite(B11110000 |  dis[6], B00100110, -1       , -1                   , dis[32], B00000100 | led[8]); // 6
		isixWrite(B11110000 |  dis[7], B00100111, -1       , -1                   , dis[32], B00000100 | led[8]); // 7

        //         *--76--*             *--84--*   *--92--*   *--76--*              *--84--*  *--92--*
		//         76543210             76543210   76543210                                   76543210		
		isixWrite(B11110000 |  dis[8], B01001000, -1       ,     32 | puls[2]     , dis[33], B00000100 | led[8]); // 8
		isixWrite(B11110000 |  dis[9], B01001001, -1       , -1                   , dis[33], B00000100 | led[8]); // 9
		isixWrite(B11110000 | dis[10], B01001010, -1       , -1                   , dis[33], B00000100 | led[8]); // 10
		isixWrite(B11110000 | dis[11], B01001011, -1       , -1                   , dis[33], B00000100 | led[8]); // 11
		
		isixWrite(B11110000 | dis[12], B10001100, -1       ,     48 | puls[3]     , dis[34], B00000100 | led[8]); // 12
		isixWrite(B11110000 | dis[13], B10001101, -1       , -1                   , dis[34], B00000100 | led[8]); // 13
		isixWrite(B11110000 | dis[14], B10001110, -1       , -1                   , dis[34], B00000100 | led[8]); // 14
		isixWrite(B11110000 | dis[15], B10001111, -1       , -1                   , dis[34], B00000100 | led[8]); // 15

        //         *--76--*             *--84--*   *--92--*   *--76--*              *--84--*  *--92--*
		//                              76543210   76543210                                   76543210		
		isixWrite(dis[16],             B00000000, B00000001,     64 | puls[4]     , dis[35], B00000100 | led[8]); // 16
		isixWrite(dis[17],             B00000001, -1       , -1                   , dis[35], B00000100 | led[8]); // 17
		isixWrite( led[0],             B00000010, -1       , -1                   , dis[35], B00000100 | led[8]); // 18
		isixWrite(dis[18],             B00000011, -1       , -1                   , dis[35], B00000100 | led[8]); // 19
		
		isixWrite(dis[19],             B00000100, B00000010, led[2] | 80 | puls[5] , led[3], B00000100 | led[8]); // 20
		isixWrite(dis[20],             B00000101, -1       , -1                    , led[3], B00000100 | led[8]); // 21
		isixWrite(dis[21],             B00000110, -1       , -1                    , led[3], B00000100 | led[8]); // 22
		isixWrite(dis[22],             B00000111, -1       , -1                    , led[3], B00000100 | led[8]); // 23

        //         *--76--*             *--84--*   *--92--*   *--76--*              *--84--*  *--92--*
		//                              76543210   76543210                                   76543210		
		isixWrite(dis[23],             B00001000, B00000100, led[4] | 96 | puls[6] , led[5], B00000100 | led[8]); // 24
		isixWrite(dis[24],             B00001001, -1       , -1                    , led[5], B00000100 | led[8]); // 25
		isixWrite(dis[25],             B00001010, -1       , -1                    , led[5], B00000100 | led[8]); // 26
		isixWrite(dis[26],             B00001011, -1       , -1                    , led[5], B00000100 | led[8]); // 27
		
		isixWrite(dis[27],             B00001100, B00001000, led[6] | 112 | puls[7], led[7], B00000100 | led[8]); // 28
		isixWrite(dis[28],             B00001101, -1       , -1                    , led[7], B00000100 | led[8]); // 29
		isixWrite(dis[29],             B00001110, -1       , -1                    , led[7], B00000100 | led[8]); // 30
		isixWrite(dis[30],             B00001111, -1       , -1                    , led[7], B00000100 | led[8]); // 31
	}

}



// ----------------------------- routine -----------------------

// Scrittura su I2C
void iWrite(byte addr, byte regis, byte value)
{
	Wire.beginTransmission(addr);
	Wire.write(regis); Wire.write(value);
	Wire.endTransmission();
}


// Scrittura multipla su I2C
void isixWrite(int value1, int value2, int value3, int value4, int value5, int value6)
{
	// Tx singole perch register address autoincrementa dopo ogni singola tx 
	
	// Disable output per evitare sciee su display e led
	//	Wire.beginTransmission(MAX1); Wire.write(84); Wire.write(B10000000); Wire.endTransmission();
	//	Wire.beginTransmission(MAX1); Wire.write(92); Wire.write(B00000000 | led[8]); Wire.endTransmission();
	// disable meglio alla fine della routine: evita sovraluminosit da persistenza su dis[30]
	
	// Cambio configurazione, attiva fino alla prox chiamata 
	if (value1>-1) { Wire.beginTransmission(MAX0); Wire.write(76); Wire.write(value1); Wire.endTransmission(); }
	if (value2>-1) { Wire.beginTransmission(MAX0); Wire.write(84); Wire.write(value2); Wire.endTransmission(); }
	if (value3>-1) { Wire.beginTransmission(MAX0); Wire.write(92); Wire.write(value3); Wire.endTransmission(); }
	
	if (value4>-1) { Wire.beginTransmission(MAX1); Wire.write(76); Wire.write(value4); Wire.endTransmission(); } 
	if (value5>-1) { Wire.beginTransmission(MAX1); Wire.write(84); Wire.write(value5); Wire.endTransmission(); }
	if (value6>-1) { Wire.beginTransmission(MAX1); Wire.write(92); Wire.write(value6); Wire.endTransmission(); }
	
	delayMicroseconds(50); // per maggiore persistenza (non molto utile)
	
	// Disable output per evitare sciee su display e led
	Wire.beginTransmission(MAX1); Wire.write(84); Wire.write(B10000000); Wire.endTransmission();
	Wire.beginTransmission(MAX1); Wire.write(92); Wire.write(B00000000 | led[8]); Wire.endTransmission();
}


// Fase lunare (fonte: http://www.voidware.com/moon_phase.htm)
int moon_phase(int y, int m, int d)
{
    // calculates the moon phase (0-7), accurate to 1 segment.
    //  0 =>  new moon.  4 => full moon.

    int c,e;
    double jd;
    int b;

    if (m < 3) { y--; m += 12; }

    ++m;
    c = 365.25*y;
    e = 30.6*m;
    jd = c+e+d-694039.09;  	// jd is total days elapsed
    jd /= 29.53;           	// divide by the moon cycle (29.53 days)
    b = jd;		   			// int(jd) -> b, take integer part of jd
    jd -= b;		   		// subtract integer part to leave fractional part of original jd
    b = jd*8 + 0.5;	   		// scale fraction from 0-8 and round by adding 0.5
    b = b & 7;		   		// 0 and 8 are the same so turn 8 into 0

	return b;
}
